@@ -1,12 +1,15 @@
 import logging
-from waveletai import constants
 import threading
-from waveletai.sessions import Session
-from waveletai.backends.hosted_backend import HostedBackend
-from waveletai.constants import ModelRegisterMode
 
 import mlflow
 
+from waveletai import constants
+from waveletai._version import VERSION as __version__
+from waveletai.backends.hosted_backend import HostedBackend
+from waveletai.constants import ModelRegisterMode
+from waveletai.sessions import Session
+from waveletai.utils import MLflowModuleRegister, set_mlflow_tracing_waveletai_store
+
 logging.basicConfig(format='[%(process)d] [%(asctime)s] %(levelname)s [%(filename)s:%(lineno)s] %(message)s',
                     level=logging.INFO)
 
@@ -24,12 +27,12 @@ You can pass this value as api_token during init() call, either by an environmen
 ANONYMOUS_API_TOKEN = constants.ANONYMOUS_API_TOKEN
 from waveletai._version import get_versions
 
-__version__ = get_versions()['version']
+__version__ = get_versions()
 
 _backend = None
 
 
-def init(name=None, pwd=None, backend=None):
+def init(name=None, pwd=None, backend=None, api_token=None):
     with __lock:
         global _backend
 
@@ -39,7 +42,7 @@ def init(name=None, pwd=None, backend=None):
             #     backend = OfflineBackend()
 
             # elif backend_name is None:
-            _backend = HostedBackend(name, pwd)
+            _backend = HostedBackend(name, pwd, api_token)
 
             # else:
             #     raise InvalidBackend(backend_name)
@@ -48,11 +51,18 @@ def init(name=None, pwd=None, backend=None):
         # return session
 
 
-def create_app(name, tags, desc):
+def close():
+    global _backend
+    if _backend:
+        _backend.close()
+
+
+def create_app(name, tags=[], desc=None):
     """
     创建应用
-    :param name: 应用名称
-    :param desc: 应用说明
+    :param name :str 应用名称
+    :param tags :list 应用标签
+    :param desc :str 应用说明
     :return: App对象
     """
     global _backend
@@ -95,10 +105,9 @@ def get_dataset(dataset_id):
 
 def download_dataset_artifact(dataset_id, path, destination):
     """
-    获取模型对象
+    下载数据集指定文件或目录到本地
     :param dataset_id:数据集ID
-    :param path:路径
-    :param type:数据集类型
+    :param path:当前文件或文件路径在数据集目录的相对位置, eg 下载单个文件: data/test.csv, 下载data文件夹: data
     :param destination:本地存储路径
     :return:
     """
@@ -106,16 +115,16 @@ def download_dataset_artifact(dataset_id, path, destination):
     return _backend.download_dataset_artifact(dataset_id, path, destination)
 
 
-def download_dataset_artifacts(dataset_id, destination):
+def download_dataset_artifacts(data_id, destination, unzip=False):
     """
-    获取模型对象
-    :param dataset_id:数据集ID
-    :param type:数据集类型
+    下载数据集/特征库全部文件到本地指定目录
+    :param data_id:数据集ID/特征库ID
     :param destination:本地存储路径
+    :param unzip:仅当当前数据ID为特征库ID时生效，是否自动解压zip数据，默认为不解压
     :return:
     """
     global _backend
-    return _backend.download_dataset_artifacts(dataset_id, destination)
+    return _backend.download_dataset_artifacts(data_id, destination, unzip)
 
 
 def upload_dataset_artifacts(dataset_id, path):
@@ -129,7 +138,7 @@ def upload_dataset_artifacts(dataset_id, path):
     return _backend.upload_dataset_artifacts(dataset_id, path)
 
 
-def create_model(name, app_id, desc='', auth_id='', git_url='', http_url=''):
+def create_model(name, app_id, desc=None, auth_id=None, git_url=None, http_url=None):
     """
     创建模型
     :param http_url: http地址 默认空
@@ -198,7 +207,7 @@ def list_models(app_id):
     return _backend.list_models(app_id)
 
 
-def update_dataset(dataset_id, name, zone, desc):
+def update_dataset(dataset_id, name, zone, desc=None):
     """
     更新数据集id
     :param dataset_id:
@@ -221,7 +230,7 @@ def delete_dataset(dataset_id):
     return _backend.delete_dataset(dataset_id)
 
 
-def update_model(model_id, app_id, name, desc):
+def update_model(model_id, app_id, name, desc=None):
     """
     更新模型集
     :param model_id:
@@ -265,7 +274,7 @@ def abandon_model_version(model_id, version):
     return _backend.abandon_model_version(model_id, version)
 
 
-def update_model_version(model_id, version, desc):
+def update_model_version(model_id, version, desc=None):
     """
     根据模型id，修改指定模型版本
     :param desc:
@@ -290,7 +299,7 @@ def list_experiments(model_id):
 def download_model_version_asset(repo_id, path, destination):
     """
     根据repo_id下载指定path(文件或文件夹)到本地的destination
-    :param model_id:
+    :param repo_id:
     :param path:
     :param destination:
     :return:
@@ -347,6 +356,7 @@ def create_experiment(name, model_id, params=None, desc=None, ):
                                       docker_args={})
 
 
+@set_mlflow_tracing_waveletai_store
 def get_experiment(experiment_id):
     """
     获取实验对象
@@ -357,36 +367,6 @@ def get_experiment(experiment_id):
     return _backend.get_experiment(experiment_id)
 
 
-# def get_mlflow_experiment(experiment_id):
-#     """
-#     获取mlflwo实验对象
-#     :param experiment_id:
-#     :return:
-#     """
-#     global _backend
-#     return _backend.get_mlflow_experiment(experiment_id)
-
-
-def create_mlflow_experiment(name, artifact_location):
-    """
-    创建mlflow实验
-    :param name:
-    :param artifact_location:
-    :return:
-    """
-    global _backend
-    return _backend.create_mlflow_experiment(name, artifact_location)
-
-
-# def create_mlflow_run():
-#     """
-#     创建mlflow run
-#     :return:
-#     """
-#     global _backend
-#     return _backend.create_mlflow_run()`
-
-
 def get_experiment_by_mlflow_experiment(experiment_id):
     """
     根据mlexperiment id 查询wai experiment
@@ -406,11 +386,180 @@ def get_experiment_by_mlflow_run(run_id):
     return _backend.get_experiment_by_mlflow_run(run_id)
 
 
+@set_mlflow_tracing_waveletai_store
 def log_metric(key, value, step=None):
-    mlflow.set_tracking_uri('waveletai-store://')
-    mlflow.log_metric(key, value, step)
+    """
+    记录实验标量
+    :param key:
+    :param value:
+    :param step:
+    :return:
+    """
+    return mlflow.log_metric(key, value, step)
 
 
+@set_mlflow_tracing_waveletai_store
+def log_metrics(metrics, step=None):
+    """
+    批量记录实验标量
+    :param metrics: Dict[str, float]
+    :param step:
+    :returns:
+    """
+    return mlflow.log_metrics(metrics, step)
+
+
+@set_mlflow_tracing_waveletai_store
 def log_param(key, value):
-    mlflow.set_tracking_uri('waveletai-store://')
-    mlflow.log_param(key, value)
+    """
+    记录实验参数
+    :param key:
+    :param value:
+    :return:
+    """
+    return mlflow.log_param(key, value)
+
+
+@set_mlflow_tracing_waveletai_store
+def log_params(params):
+    """
+    批量记录参数
+    :param params: Dict[str, Any]
+    :return:
+    """
+    return mlflow.log_params(params)
+
+
+@set_mlflow_tracing_waveletai_store
+def set_tag(key, value):
+    """
+    记录标签
+    :param key:
+    :param value:
+    :return:
+    """
+    return mlflow.set_tag(key, value)
+
+
+@set_mlflow_tracing_waveletai_store
+def set_tags(tags):
+    """
+    批量记录标签
+    :param tags: Dict[str, Any]
+    :return:
+    """
+    return mlflow.set_tags(tags)
+
+
+def download_feature_zip(feature_id, destination, unzip=False):
+    """
+    通过特征id下载特征zip包到本地指定目录
+    :param unzip: 是否解压 默认不解压
+    :param feature_id:特征id
+    :param destination:本地存储地址 默认空
+    :return:
+    """
+    global _backend
+    return _backend.download_feature_zip(feature_id, destination, unzip)
+
+
+def upload_artifact(local_file, artifact_path, run_id):
+    """
+    上传模型文件
+    :param local_file:
+    :param artifact_path:
+    :param run_id:
+    :return:
+    """
+    global _backend
+    return _backend.upload_artifact(local_file, artifact_path, run_id)
+
+
+@set_mlflow_tracing_waveletai_store
+def log_batch(metrics=None, params=None, tags=None):
+    """
+    批量上传 metric param tag
+    :param metrics:
+    :param params:
+    :param tags:
+    :return:
+    """
+    if metrics is None:
+        metrics = []
+    if params is None:
+        params = []
+    if tags is None:
+        tags = []
+    from mlflow.tracking.fluent import _get_or_start_run, MlflowClient
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_batch(run_id, metrics, params, tags)
+    _logger.warning(f'成功记录metric {len(metrics)}个, param {len(params)}个, tag {len(tags)}个')
+
+
+@set_mlflow_tracing_waveletai_store
+def log_artifact(local_path: str, artifact_path=None):
+    """
+    上传artifact
+    :param local_path:
+    :param artifact_path:
+    :return:
+    """
+    return mlflow.log_artifact(local_path=local_path, artifact_path=artifact_path)
+
+
+@set_mlflow_tracing_waveletai_store
+def log_artifacts(local_dir: str, artifact_path=None):
+    """
+    上传artifact文件夹
+    :param local_dir:
+    :param artifact_path:
+    :return:
+    """
+    return mlflow.log_artifacts(local_dir=local_dir, artifact_path=artifact_path)
+
+
+@set_mlflow_tracing_waveletai_store
+def auto_log(log_input_examples: bool = False,
+             log_model_signatures: bool = True,
+             log_models: bool = True,
+             disable: bool = False,
+             exclusive: bool = False,
+             disable_for_unsupported_versions: bool = False,
+             silent: bool = False):
+    """
+    上传artifact文件夹
+     :param log_input_examples: If ``True``, input examples from training datasets are collected and
+                               logged along with model artifacts during training. If ``False``,
+                               input examples are not logged.
+                               Note: Input examples are MLflow model attributes
+                               and are only collected if ``log_models`` is also ``True``.
+    :param log_model_signatures: If ``True``,
+                                 :py:class:`ModelSignatures <mlflow.models.ModelSignature>`
+                                 describing model inputs and outputs are collected and logged along
+                                 with model artifacts during training. If ``False``, signatures are
+                                 not logged. Note: Model signatures are MLflow model attributes
+                                 and are only collected if ``log_models`` is also ``True``.
+    :param log_models: If ``True``, trained models are logged as MLflow model artifacts.
+                       If ``False``, trained models are not logged.
+                       Input examples and model signatures, which are attributes of MLflow models,
+                       are also omitted when ``log_models`` is ``False``.
+    :param disable: If ``True``, disables all supported autologging integrations. If ``False``,
+                    enables all supported autologging integrations.
+    :param exclusive: If ``True``, autologged content is not logged to user-created fluent runs.
+                      If ``False``, autologged content is logged to the active fluent run,
+                      which may be user-created.
+    :param disable_for_unsupported_versions: If ``True``, disable autologging for versions of
+                      all integration libraries that have not been tested against this version
+                      of the MLflow client or are incompatible.
+    :param silent: If ``True``, suppress all event logs and warnings from MLflow during autologging
+                   setup and training execution. If ``False``, show all events and warnings during
+                   autologging setup and training execution.
+    :return:
+    """
+    return mlflow.autolog(log_input_examples,
+                          log_model_signatures,
+                          log_models,
+                          disable,
+                          exclusive,
+                          disable_for_unsupported_versions,
+                          silent)
