@@ -156,17 +156,21 @@ class Backend(object):
         pass
 
     @abstractmethod
-    def log_param(self, run_id, params):
+    def log_params(self, run_id, params):
         pass
 
     @abstractmethod
-    def log_metric(self, run_id, metric, value, is_nan):
+    def log_metrics(self, run_id, metric, value, is_nan):
         pass
 
     @abstractmethod
-    def set_tag(self, run_id, tag):
+    def set_tags(self, run_id, tags):
         pass
 
     @abstractmethod
     def update_run_info(self, run_id, run_status, end_time):
         pass
+
+    @abstractmethod
+    def upload_artifact(self, local_file, artifact_path, run_id):
+        pass
