@@ -8,6 +8,8 @@
 @File    : hosted_backend.py
 @Desc    :
 """
+import base64
+import json
 import logging
 import io
 import os
@@ -23,19 +25,21 @@ from waveletai.backend import Backend
 from waveletai.utils import with_api_exceptions_handler
 from waveletai.oauth import WaveletAIAuthenticator
 from waveletai.dataset import Dataset, Asset
+from waveletai.feature import Feature
 from waveletai.model import ModelVersion
 from waveletai.app import App
-from waveletai.exceptions import LoginFailed, NotADirectory, MissingNameOrPwdException, FileUploadException
+from waveletai.exceptions import LoginFailed, NotADirectory, MissingNameOrPwdException, FileUploadException, \
+    InvalidAPIToken
 from waveletai.constants import DataType, ModelRegisterMode
 from waveletai.utils.storage_utils import UploadEntry, scan_upload_entries, SilentProgressIndicator, \
     LoggingProgressIndicator, upload_to_storage, dataset_upload_to_storage
 from waveletai.utils.datastream import FileStream, FileChunkStream, FileChunk
-from waveletai.experiment import Experiment, MLFlowExperiment, MLFlowRun
+import zipfile
+from waveletai.experiment import Experiment, MLFlowExperiment, MLFlowRun, MLflowLatestMetric, MLFlowParam, MLFlowTag
 
 _logger = logging.getLogger(__name__)
 
 
-
 class HostedBackend(Backend):
 
     @with_api_exceptions_handler
@@ -144,7 +148,7 @@ class HostedBackend(Backend):
         return data
 
     @with_api_exceptions_handler
-    def update_model(self, model_id, app_id, name, desc=''):
+    def update_model(self, model_id, app_id, name, desc):
         """
         更新模型信息
         :param app_id:
@@ -292,12 +296,13 @@ class HostedBackend(Backend):
         return data
 
     @with_api_exceptions_handler
-    def __init__(self, name=None, pwd=None):
+    def __init__(self, name=None, pwd=None, api_token=None):
         if name is None:
             name = os.getenv(envs.USER_NAME)
         if pwd is None:
             pwd = os.getenv(envs.USER_PWD)
-        api_token = os.getenv(envs.API_TOKEN)
+        if not api_token:
+            api_token = os.getenv(envs.API_TOKEN)
         if all([name, pwd]) or api_token:
             pass
         else:
@@ -319,11 +324,51 @@ class HostedBackend(Backend):
             if res.json()["message"]:
                 raise LoginFailed(res.json()["message"])
             print("WaveletAI Backend connected")
-            api_token = res.json()["data"]["token"]
-            os.environ.update({envs.API_TOKEN: api_token})
+            data = res.json()['data']
+            api_token = data["token"]
+            user_id = data['user_id']
+            os.environ.update({
+                envs.API_TOKEN: api_token,
+                'LOGNAME': user_id
+            })
+        elif len(api_token) > 32:
+            # api-token登录
+            try:
+                od = json.loads(base64.b64decode(api_token).decode('utf-8'))
+                os.environ.update({
+                    envs.API_URL: od['api_url']
+                })
+            except Exception as e:
+                raise InvalidAPIToken()
+            res = self._session.post(
+                url=f'{self.api_address}/account/users/sdk/login/',
+                json={'api_token': api_token},
+                headers=headers,
+            )
+            if res.json()["message"]:
+                raise LoginFailed(res.json()["message"])
+            print("WaveletAI Backend connected")
+            data = res.json()['data']
+            api_token = data["token"]
+            user_id = data['user_id']
+            os.environ.update({
+                envs.API_TOKEN: api_token,
+                'LOGNAME': user_id
+            })
 
         self._session.headers.update({"X-TOKEN": api_token})
 
+    def close(self):
+        """
+        清理数据, 关闭连接
+        :return:
+        """
+        clean_key = [envs.API_TOKEN, 'LOGNAME']
+        for k in clean_key:
+            if os.environ.get(k):
+                os.environ.pop(k)
+        self._session.close()
+
     @with_api_exceptions_handler
     def _create_authenticator(self, api_token, ssl_verify):
         return WaveletAIAuthenticator(api_token, ssl_verify)
@@ -345,7 +390,7 @@ class HostedBackend(Backend):
     def get_dataset(self, dataset_id):
         res = self._session.get(url=f'{self.api_address}/data/dataset/{dataset_id}')
         if res.json()["message"]:
-            raise Exception(res.json()["message"])
+            return None
         dict = res.json()['data']
         return Dataset(self, dict['id'], dict['name'], dict['desc'], dict['zone'], dict['dimension'], dict['json_data'],
                        dict['create_time'], dict['type'], dict['create_user_id'], dict['update_time'],
@@ -507,9 +552,14 @@ class HostedBackend(Backend):
     def _url_download(self, urls):
         url, destination, filename = urls
         destination_loc = os.path.join(destination, filename)
+        content_size = 0
+        headers_types = ['application/octet-stream', 'application/zip', 'image/png', 'image/jpeg']
         with closing(requests.get(url, stream=True)) as response:
-            # content_size = int(response.headers['content-length'])  # 内容体总大小
-            content_size = len(response.content)
+            if response.headers.get('Content-Type') not in headers_types:
+                _logger.error(f"当前Url不是可下载的文件流地址，请检查后再试")
+                return
+            if 'Content-Length' in response.headers:
+                content_size = int(response.headers.get('Content-Length'))  # 内容体总大小
             progress_indicator = SilentProgressIndicator(content_size, filename)
             if content_size >= WARN_SIZE:
                 progress_indicator = LoggingProgressIndicator(content_size, filename)
@@ -559,13 +609,10 @@ class HostedBackend(Backend):
                     if '.' not in path:
                         resultinfo, dir = self._list_dir_artifacts(dataset_id, path, [], [])
                         for i in dir:
-                            print(i)
                             if not os.path.exists(destination + i):
                                 os.makedirs(destination + i)
                         for asset in artifacts:
-                            print(asset.name)
                             for res in resultinfo:
-                                print(res)
                                 if asset.name in res:
                                     if '/' in res:
                                         res_des = res.rsplit('/', 1)
@@ -584,16 +631,24 @@ class HostedBackend(Backend):
             raise e
 
     @with_api_exceptions_handler
-    def download_dataset_artifacts(self, dataset_id, destination):
+    def download_dataset_artifacts(self, data_id, destination, unzip):
         """
-        :param dataset_id:
+        :param data_id:
         :param destination:
+        :param unzip:
         :return:
         """
-        dataset = self.get_dataset(dataset_id)
-        dataset_type = dataset.data_type
-        name = dataset.name
-        self.download_dataset_artifacts_(dataset_id, dataset_type, name, destination)
+        dataset = self.get_dataset(data_id)
+
+        if dataset is not None:
+            dataset_type = dataset.data_type
+            name = dataset.name
+            self.download_dataset_artifacts_(data_id, dataset_type, name, destination)
+        else:
+            feature = self.get_feature(data_id)
+            self._download_feature_zip(data_id, destination, unzip)
+            if feature is None:
+                raise Exception('需下载的数据集/特征库ID未找到，请检查后重新尝试')
 
     @with_api_exceptions_handler
     def download_dataset_artifacts_(self, dataset_id, type, name, destination):
@@ -706,7 +761,7 @@ class HostedBackend(Backend):
             return e
 
     @with_api_exceptions_handler
-    def create_model(self, app_id, name, desc='', auth_id='', git_url='', http_url=''):
+    def create_model(self, app_id, name, desc, auth_id, git_url, http_url):
         """
         当前应用下创建模型
         :param app_id: 应用ip
@@ -726,7 +781,9 @@ class HostedBackend(Backend):
         if res.json()['message']:
             raise Exception(res.json()['message'])
         data = res.json()['data']
-        return data
+        return Model(self, data['id'], data['name'], data['desc'], data['app_id'],
+                     data['create_time'], data['create_user_id'], data['update_time'],
+                     data['update_user_id'])
 
     @with_api_exceptions_handler
     def get_model(self, model_id):
@@ -1009,6 +1066,47 @@ class HostedBackend(Backend):
         return ModelVersion(self, data['id'], data['version'], data['desc'], data['model_id'], data['model_name'],
                             data['create_user_name'], data['create_time'], data['create_user_id'], data['mode'])
 
+    @with_api_exceptions_handler
+    def _get_feature_zip_path(self, feature_id):
+        res = self._session.get(url=f'{self.api_address}/data/feature/{feature_id}/asset')
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        data = res.json()['data']
+        feature_path = ''
+        if data['path'] is not None:
+            feature_path = data['path']
+        return feature_path
+
+    @with_api_exceptions_handler
+    def _download_feature_zip(self, feature_id, destination, unzip):
+        """
+        :param unzip: 是否解压
+        :param feature_id:特征id
+        :param destination:本地存储路径
+        :return:
+        """
+        if not destination:
+            destination = os.getcwd()
+
+        if not os.path.exists(destination):
+            os.makedirs(destination)
+        elif not os.path.isdir(destination):
+            raise NotADirectory(destination)
+        feature_path = self._get_feature_zip_path(feature_id)
+        if feature_path is None:
+            return '该特征下无可下载资源'
+        try:
+            file_name = feature_path.rsplit('/', 1)
+            file_name = file_name[-1]
+            self._url_download((feature_path, destination, file_name))
+            zip_path = destination + '/' + file_name
+            if unzip is True:
+                with zipfile.ZipFile(zip_path, 'r') as zip_ref:
+                    zip_ref.extractall(destination)
+                os.remove(zip_path)
+        except Exception as e:
+            raise e
+
     @with_api_exceptions_handler
     def get_experiment(self, experiment_id):
         """
@@ -1239,77 +1337,47 @@ class HostedBackend(Backend):
                          )
 
     @with_api_exceptions_handler
-    def log_param(self, run_id, param):
+    def log_params(self, run_id, params):
         """
         log param
         :param run_id:
-        :param param:
+        :param params:
         :return:
         """
-        data = {
-            'key': param.key,
-            'value': param.value
-        }
-        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_param', json=data)
+        data = []
+        for param in params:
+            data.append({'key': param.key, 'value': param.value})
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_params', json={'data': data})
         if res.json()['message']:
             raise Exception(res.json()['message'])
 
     @with_api_exceptions_handler
-    def log_metric(self, run_id, metric, value, is_nan):
+    def log_metrics(self, run_id, metrics):
         """
         log metric
         :param run_id:
-        :param metric:
+        :param metrics: List[Dict[metric]]
         :return:
         """
-        data = {
-            'key': metric.key,
-            'value': value,
-            'timestamp': metric.timestamp,
-            'step': metric.step,
-            'is_nan': is_nan
-        }
-        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_metric', json=data)
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/log_metrics', json={'data': metrics})
         if res.json()['message']:
             raise Exception(res.json()['message'])
 
     @with_api_exceptions_handler
-    def set_tag(self, run_id, tag):
+    def set_tags(self, run_id, tags):
         """
         log tag
         :param run_id:
-        :param tag:
+        :param tags:
         :return:
         """
-        data = {
-            'key': tag.key,
-            'value': tag.value
-        }
-        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/set_tag', json=data)
+        data = []
+        for tag in tags:
+            data.append({'key': tag.key, 'value': tag.value})
+        res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}/set_tags', json={'data': data})
         if res.json()['message']:
             raise Exception(res.json()['message'])
 
-    # @with_api_exceptions_handler
-    # def log_batch(self, run_id, metrics, params, tags):
-    #     """
-    #     写入mlflow log
-    #     :param run_id:
-    #     :param metrics:
-    #     :param params:
-    #     :param tags:
-    #     :return:
-    #     """
-    #     data = {
-    #         'metrics': metrics,
-    #         'params': params,
-    #         'tags': tags
-    #     }
-    #     res = self._session.post(url=f'{self.api_address}/training/mlflow/run/{run_id}/log_batch')
-    #     if res.json()['message']:
-    #         raise Exception(res.json()['message'])
-    #     data = res.json()['data']
-    #     return
-
     @with_api_exceptions_handler
     def update_run_info(self, run_id, run_status, end_time):
         """
@@ -1329,3 +1397,127 @@ class HostedBackend(Backend):
         data = res.json()['data']
 
         return MLFlowRun(self, **data)
+
+    @with_api_exceptions_handler
+    def _upload_artifact(self, entry: UploadEntry, model_repo_id):
+        fs = FileStream(entry)
+        progress_indicator = SilentProgressIndicator(fs.length, fs.filename)
+        payload = {"path": entry.prefix, "run_id": model_repo_id}
+        res = self._session.post(url=f'{self.api_address}/training/mlflow/upload/artifact', files=[('file', (
+            fs.filename, fs.fobj, fs.content_type))], data=payload)
+        if res.json()["message"]:
+            _logger.error(f"file {entry.source_path} upload failed,{res.json()['message']}")
+            raise FileUploadException(fs.filename)
+        else:
+            progress_indicator.complete()
+        fs.close()
+
+    @with_api_exceptions_handler
+    def _upload_chunk_artifact_loop(self, entry: UploadEntry, model_repo_id):
+        stream = FileChunkStream(entry)
+        progress_indicator = LoggingProgressIndicator(stream.length, stream.filename)
+        for ind, fc in enumerate(stream.generate(chunk_size=CHUNK_SIZE * 1024 * 20)):
+            self._upload_chunk_artifact(fc, entry.filename, str(ind), entry.prefix, progress_indicator, model_repo_id)
+
+        res = self._session.get(url=f'{self.api_address}/training/mlflow/upload/artifact/success',
+                                params={'filename': entry.filename, 'run_id': model_repo_id, 'path': entry.prefix})
+        if res.json()["message"]:
+            _logger.error(f"file {entry.source_path} upload failed,{res.json()['message']}")
+            raise FileUploadException(entry.filename)
+        else:
+            progress_indicator.complete()
+        stream.close()
+
+    @with_api_exceptions_handler
+    def _upload_chunk_artifact(self, file: FileChunk, filename, chunk, path, progress_indicator, model_repo_id):
+        payload = {"chunk": chunk, "path": path, "run_id": model_repo_id}
+        res = self._session.post(url=f'{self.api_address}/training/mlflow/upload/artifact',
+                                 files=[('file', (file.data))],
+                                 data=payload
+                                 )
+        if res.json()["message"]:
+            _logger.error(f"file {filename} upload failed,{res.json()['message']}")
+            # raise FileUploadException(filename + '_' + chunk)
+        else:
+            progress_indicator.progress(file.end - file.start)
+
+    @with_api_exceptions_handler
+    def upload_artifact(self, local_file, artifact_path, run_id):
+        """
+        模型文件上传接口
+        :param local_file:
+        :param artifact_path:
+        :param run_id:
+        :return:
+        """
+        entries = scan_upload_entries({UploadEntry(local_file, artifact_path)})
+        upload_to_storage(entries, self._upload_artifact, self._upload_chunk_artifact_loop,
+                          warn_limit=CHUNK_SIZE * 1024 * 50, model_repo_id=run_id)
+
+    # @with_api_exceptions_handler
+    # def record_logged_model(self, key, value, run_id):
+    #     """
+    #     记录模型保存记录
+    #     :param key:
+    #     :param value:
+    #     :param run_id:
+    #     :return:
+    #     """
+    #     data = {
+    #         'key': key,
+    #         'value': value
+    #     }
+    #     res = self._session.post(f'{self.api_address}/training/mlflow/run/{run_id}', json=data)
+    #     if res.json()['message']:
+    #         raise Exception(res.json()['message'])
+
+    @with_api_exceptions_handler
+    def get_latest_metrics(self, run_id):
+        """
+        获取mlflow的运行标量
+        :param run_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/mlflow/run/{run_id}/latest_metric/')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return [MLflowLatestMetric(**d) for d in data]
+
+    @with_api_exceptions_handler
+    def get_params(self, run_id):
+        """
+        获取mlflow的运行参数
+        :param run_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/mlflow/run/{run_id}/param/')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return [MLFlowParam(**d) for d in data]
+
+    @with_api_exceptions_handler
+    def get_tags(self, run_id):
+        """
+        获取mlflow的运行的标签
+        :param run_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/mlflow/run/{run_id}/tag/')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+
+        return [MLFlowTag(**d) for d in data]
+
+    @with_api_exceptions_handler
+    def get_feature(self, feature_id):
+        res = self._session.get(url=f'{self.api_address}/data/feature/{feature_id}/')
+        if res.json()["message"]:
+            return None
+        dict = res.json()['data']['data']
+        return Feature(self, dict['id'], dict['name'], dict['desc'], dict['zone'], dict['type'], dict['json_data'],
+                       dict['create_time'], dict['create_user_id'], dict['update_time'], dict['update_user_id'])
