@@ -9,7 +9,7 @@
 @Desc    : 
 """
 
-"""{"api_url":"http://ai.xiaobodata.com","api_token":"","api_key":"1e1cad40-7605-4a64-bd74-151226ddc53a"}"""
+"""{"api_url":"http://ai.xiaobodata.com","api_token":"","api_key":""}"""
 from enum import Enum, unique
 
 ANONYMOUS = 'ANONYMOUS'
